"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.BleComboLLSyncDeviceAdapter = exports.BleComboEspDeviceAdapter = exports.BleComboDualModeDeviceAdapter4H5 = exports.BleComboDualModeDeviceAdapter = void 0;
const qcloud_iotexplorer_appdev_plugin_wificonf_core_1 = require("qcloud-iotexplorer-appdev-plugin-wificonf-core");
const qcloud_iotexplorer_appdev_sdk_1 = require("qcloud-iotexplorer-appdev-sdk");
const protocols_1 = require("./protocols");
const qcloud_iotexplorer_common_libs_1 = require("qcloud-iotexplorer-common-libs");
const { delay, } = qcloud_iotexplorer_appdev_sdk_1.AppDevSdk.utils;
const { WifiConfStepCode, WifiConfStepDesp, WifiConfErrorMsg, } = qcloud_iotexplorer_appdev_plugin_wificonf_core_1.constants;
var protocols_2 = require("./protocols");
Object.defineProperty(exports, "BleComboDualModeDeviceAdapter", { enumerable: true, get: function () { return protocols_2.BleComboDualModeDeviceAdapter; } });
Object.defineProperty(exports, "BleComboDualModeDeviceAdapter4H5", { enumerable: true, get: function () { return protocols_2.BleComboDualModeDeviceAdapter4H5; } });
Object.defineProperty(exports, "BleComboEspDeviceAdapter", { enumerable: true, get: function () { return protocols_2.BleComboEspDeviceAdapter; } });
Object.defineProperty(exports, "BleComboLLSyncDeviceAdapter", { enumerable: true, get: function () { return protocols_2.BleComboLLSyncDeviceAdapter; } });
const getBleComboProtoHandler = (taskOptions) => {
    switch (taskOptions.wifiConfType) {
        case "ble" /* WifiConfType.BLEComboEsp */:
            return new protocols_1.BleComboEspTask(taskOptions);
        case "llsyncble" /* WifiConfType.BLEComboLLSync */:
        case "dualmode" /* WifiConfType.DualMode */:
            return new protocols_1.BleComboLLSyncTask(taskOptions);
        default:
            console.warn('没有找到 wifiConfType 对应的 bleComboHandler:', taskOptions.wifiConfType);
    }
};
const protocolNameMap = {
    ["ble" /* WifiConfType.BLEComboEsp */]: 'ESP',
    ["llsyncble" /* WifiConfType.BLEComboLLSync */]: 'LLSync',
    ["dualmode" /* WifiConfType.DualMode */]: 'LLSync Dualmode',
};
class BleComboPlugin {
    static install(sdk) {
        sdk.plugins.wifiConfBleCombo = {
            async start(options) {
                var _a, _b;
                const { onError, onProgress, onComplete, wifiConfToken, familyId, roomId = '', wifiConfType, } = options;
                const stepCheck = async (duration = 1000) => {
                    await delay(duration);
                };
                const flowLogger = new qcloud_iotexplorer_common_libs_1.FlowLogger('ble.combo').flow(options.wifiConfType, `${protocolNameMap[wifiConfType] || wifiConfType} BLE Combo 配网`);
                // 先连接再 start，这里尝试从 deviceAdapter 里面拿 flowId，把两部分日志关联起来
                if ((_a = options.deviceAdapter) === null || _a === void 0 ? void 0 : _a.flowId) {
                    flowLogger.context.flowId = (_b = options.deviceAdapter) === null || _b === void 0 ? void 0 : _b.flowId;
                }
                flowLogger.externalContextGetter = () => {
                    var _a, _b, _c;
                    return ({
                        productId: (_a = options.deviceAdapter) === null || _a === void 0 ? void 0 : _a.productId,
                        deviceName: (_b = options.deviceAdapter) === null || _b === void 0 ? void 0 : _b.deviceName,
                        bleDeviceId: (_c = options.deviceAdapter) === null || _c === void 0 ? void 0 : _c.deviceId,
                    });
                };
                flowLogger.start();
                // step 1
                onProgress({
                    code: "PROTOCOL_START" /* WifiConfStepCode.PROTOCOL_START */,
                    reportEvents: ['配网', '发送wifi'],
                });
                try {
                    const startTime = Date.now();
                    const bleComboTask = getBleComboProtoHandler(Object.assign(Object.assign({}, options), { reporter: {
                            info(message, data = {}) {
                                onProgress({
                                    code: "PROTOCOL_DETAIL" /* WifiConfStepCode.PROTOCOL_DETAIL */,
                                    detail: {
                                        message: `${"PROTOCOL_DETAIL" /* WifiConfStepCode.PROTOCOL_DETAIL */}(${WifiConfStepDesp["PROTOCOL_DETAIL" /* WifiConfStepCode.PROTOCOL_DETAIL */]}: ${message})`,
                                        data,
                                    },
                                });
                            },
                            error(error) {
                                onError({
                                    code: 'PROTOCOL_FAIL',
                                    detail: { error },
                                });
                            },
                        }, flowLogger }));
                    try {
                        await flowLogger
                            .step('bleSendWifiInfo', '蓝牙发送 WiFi 信息', true)
                            .wrap(bleComboTask.start());
                    }
                    catch (err) {
                        console.error(err);
                        throw Object.assign({ code: 'PROTOCOL_FAIL' }, err);
                    }
                    // STEP 2
                    onProgress({
                        code: "PROTOCOL_SUCCESS" /* WifiConfStepCode.PROTOCOL_SUCCESS */,
                        detail: {
                            timeCost: Date.now() - startTime,
                        },
                        reportEvents: ['发送wifi'],
                    });
                    await stepCheck();
                    onProgress({ code: "BUSINESS_START" /* WifiConfStepCode.BUSINESS_START */ });
                    onProgress({
                        code: 'BLE_SEND_TOKEN_START',
                        detail: {
                            data: {
                                wifiConfToken,
                            },
                        },
                        reportEvents: ['bleSendToken'],
                    });
                    const startSendToken = Date.now();
                    let response;
                    try {
                        response = await flowLogger
                            .step('bleSendToken', '蓝牙发送配网 Token', true)
                            .wrap(bleComboTask.sendToken());
                    }
                    catch (error) {
                        throw Object.assign({ code: 'BLE_SEND_TOKEN_ERROR' }, error);
                    }
                    await onProgress({
                        code: 'BLE_SEND_TOKEN_SUCCESS',
                        detail: {
                            timeCost: Date.now() - startSendToken,
                            response,
                        },
                        reportEvents: ['bleSendToken'],
                    });
                    /**
                   * 这里要判断是不是双路通信，如果是，需要
                   * 1. 查询token
                   * 2. 绑定蓝牙
                   * 3. 到后台绑定设备
                   */
                    const { productId, deviceName } = response;
                    if (options.wifiConfType === "dualmode" /* WifiConfType.DualMode */) {
                        try {
                            await flowLogger
                                .step('queryToken', '轮询配网 Token', true)
                                .wrap((0, qcloud_iotexplorer_appdev_plugin_wificonf_core_1.queryTokenStateAndBind)({
                                token: options.wifiConfToken,
                                productId,
                                deviceName,
                                familyId,
                                roomId,
                                onProgress,
                                sdk,
                                skipBind: true,
                                newTokenVersion: false,
                            }));
                            await flowLogger
                                .step('bleBindDevice', '蓝牙绑定设备', true)
                                .wrap(options.deviceAdapter.bindDevice({
                                familyId,
                                roomId,
                            }));
                        }
                        catch (err) {
                            throw (Object.assign({ code: 'BLE_BIND_DEVICE_ERROR' }, err));
                        }
                    }
                    else {
                        await flowLogger
                            .step('queryToken', '轮询配网 Token', true)
                            .wrap((0, qcloud_iotexplorer_appdev_plugin_wificonf_core_1.queryTokenStateAndBind)({
                            token: options.wifiConfToken,
                            productId,
                            deviceName,
                            familyId,
                            roomId,
                            onProgress,
                            sdk,
                            newTokenVersion: options.newTokenVersion,
                        }));
                    }
                    // STEP 3
                    await onProgress({
                        code: "BUSINESS_QUERY_TOKEN_STATE_SUCCESS" /* WifiConfStepCode.BUSINESS_QUERY_TOKEN_STATE_SUCCESS */,
                        detail: {
                            timeCost: Date.now() - startSendToken,
                            response,
                        },
                    });
                    await stepCheck();
                    onComplete({ productId, deviceName });
                    flowLogger.success();
                    // 成功了之后，断开连接，否则不断开，还需要上传设备端日志的
                    bleComboTask.interrupt();
                    // STEP 4
                    onProgress({
                        code: "WIFI_CONF_SUCCESS" /* WifiConfStepCode.WIFI_CONF_SUCCESS */,
                        detail: {
                            timeCost: Date.now() - startTime,
                        },
                        reportEvents: [
                            '配网',
                        ],
                    });
                }
                catch (error) {
                    flowLogger.fail(null, error);
                    if (error && error.code in WifiConfErrorMsg) {
                        error.uiMsg = WifiConfErrorMsg[error.code];
                    }
                    // 单独传出去上报一下
                    onProgress({
                        code: "WIFI_CONF_FAIL" /* WifiConfStepCode.WIFI_CONF_FAIL */,
                        detail: { error },
                    });
                    // code === WIFI_CONF_FAIL，不会被小程序上报
                    onError({
                        code: "WIFI_CONF_FAIL" /* WifiConfStepCode.WIFI_CONF_FAIL */,
                        detail: { error },
                    });
                }
            },
        };
    }
}
exports.default = BleComboPlugin;
//# sourceMappingURL=index.js.map