"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.BleComboDualModeDeviceAdapter4H5 = exports.BleComboDualModeDeviceAdapter = exports.BleComboDualModeDeviceAdapterBase = void 0;
/* eslint-disable @typescript-eslint/naming-convention */
const BleComboLLSync_1 = require("../BleComboLLSync");
const qcloud_iotexplorer_bluetooth_adapter_1 = require("qcloud-iotexplorer-bluetooth-adapter");
const qcloud_iotexplorer_bluetooth_adapter_llsync_1 = require("qcloud-iotexplorer-bluetooth-adapter-llsync");
const { hex2str } = qcloud_iotexplorer_bluetooth_adapter_llsync_1.utils;
const { REPORT_EVENT_TYPE } = qcloud_iotexplorer_bluetooth_adapter_llsync_1.constants;
const { arrayBufferToHexStringArray } = qcloud_iotexplorer_bluetooth_adapter_1.blueToothHelper;
const DUALMODE_SERVICEID = '0000FFE8-65D0-4E20-B56A-E493541BA4E2';
const DUALMODE_SERVICEID16 = '0000FFE8-0000-1000-8000-00805F9B34FB';
class LLSyncDualModeDeviceAdapter extends qcloud_iotexplorer_bluetooth_adapter_llsync_1.LLSyncDeviceAdapter {
}
LLSyncDualModeDeviceAdapter.serviceId16 = DUALMODE_SERVICEID16; // 搜索时匹配的serviceId
LLSyncDualModeDeviceAdapter.serviceId = DUALMODE_SERVICEID; // 服务的serviceId
class BleComboDualModeDeviceAdapterBase extends BleComboLLSync_1.BleComboLLSyncDeviceAdapterBase {
    static injectOptions(options) {
        exports.BleComboDualModeDeviceAdapter.options = options;
        qcloud_iotexplorer_bluetooth_adapter_llsync_1.LLSyncDeviceAdapter.injectOptions(options);
    }
    constructor(props) {
        super(props);
        this._bleDeviceAdapter = new LLSyncDualModeDeviceAdapter(props);
        // h5的 write 和小程序的 write 不一样
        this._bleDeviceAdapter.write = this.write.bind(this._bleDeviceAdapter);
        // 将消息转发到标准蓝牙adapter
        [
            'PROPERTY_REPORT',
            'CONTROL_REPLY',
            'GET_STATUS',
            'EVENT_REPORT',
            'ACTION_REPLY',
            'BIND_AUTH',
            'CONNECT_AUTH',
            'UNBIND_AUTH',
            'DEVICE_INFO',
            'UPDATE_REPLY',
            'UPDATE_DATA_REPLY',
            'UPDATE_DATA_CHECK_REPLY',
            'MTU_MODULE_CALLBACK',
            'USER_CHECK_TIMEOUT_CALLBACK',
            'REGISTER_DEVICE_INFO_REPLY',
        ].forEach((evtName) => {
            this.on(evtName, (...payload) => {
                console.info(`[dualmodeDeviceAdapter] emit ${evtName} from dualmode adapter`, payload);
                this._bleDeviceAdapter.emit(evtName, ...payload);
            });
        });
        // 蓝牙连接时，改变_bleDeviceAdapter状态
        this.on('connect', (deviceInfo) => {
            console.info('ble connected', deviceInfo);
            this._bleDeviceAdapter._deviceConnected = true;
            this._bleDeviceAdapter.emit('connect');
        });
        this.on('disconnect', (deviceInfo) => {
            console.info('ble disconnect', deviceInfo);
            this._bleDeviceAdapter._deviceConnected = false;
            this._bleDeviceAdapter.emit('disconnect');
        });
    }
    toString() {
        return '[BleComboDualModeDeviceAdapter]';
    }
    // @ts-ignore
    get authorized() {
        return this._bleDeviceAdapter.authorized;
    }
    authenticateConnection(params) {
        return this._bleDeviceAdapter.authenticateConnection(params);
    }
    bindDevice(data) {
        return this._bleDeviceAdapter.bindDevice(data);
    }
    unbindDevice(data) {
        return this._bleDeviceAdapter.unbindDevice.call(this._bleDeviceAdapter, data);
    }
    controlDevice(data) {
        return this._bleDeviceAdapter.controlDevice.call(this._bleDeviceAdapter, data);
    }
    init() {
        return this._bleDeviceAdapter.init();
    }
}
exports.BleComboDualModeDeviceAdapterBase = BleComboDualModeDeviceAdapterBase;
BleComboDualModeDeviceAdapterBase.serviceId16 = DUALMODE_SERVICEID16; // 搜索时匹配的serviceId
BleComboDualModeDeviceAdapterBase.serviceId = DUALMODE_SERVICEID; // 服务的serviceId
BleComboDualModeDeviceAdapterBase.deviceFilter = (device, extendInfo) => {
    var _a, _b, _c;
    if (!device.advertisServiceUUIDs
        || !device.advertisServiceUUIDs.find(id => id === exports.BleComboDualModeDeviceAdapter.serviceId16)
        || !device.advertisData) {
        return null;
    }
    try {
        // 计算是否要上报，这里取样1/10
        const shouldReport = Math.floor(Math.random() * 10) % 10 === 1 || true;
        const hexArr = arrayBufferToHexStringArray(device.advertisData);
        if (shouldReport) {
            (_a = exports.BleComboDualModeDeviceAdapter.options.appDevSdk.reporter) === null || _a === void 0 ? void 0 : _a.info(REPORT_EVENT_TYPE, {
                message: '收到广播',
                serviceId: exports.BleComboDualModeDeviceAdapter.serviceId,
                hexArr: hexArr.join(','),
            });
        }
        const bindStateId = parseInt(hexArr[2], 16) - ((parseInt(hexArr[2], 16) >> 2) << 2);
        const moduleVersion = parseInt(hexArr[2], 16) >> 4;
        // 目标deviceId
        const targetDeviceId = extendInfo.productId && extendInfo.deviceName
            ? `${extendInfo.productId}/${extendInfo.deviceName}`
            : '';
        // 目标设备标识
        const targetDeviceIdentify = targetDeviceId
            ? qcloud_iotexplorer_bluetooth_adapter_llsync_1.utils.get8ByteFromStr(`${targetDeviceId.replace('/', '')}`)
            : '';
        // const currentUserIdentify = LLSyncDeviceAdapter.getUserIdentify(); // 当前登陆用户标识
        const isSearchSpecificDeviceMode = !!targetDeviceId;
        let deviceProductId = '';
        let deviceUserIdentify = '';
        let deviceIdentify = '';
        const bindState = qcloud_iotexplorer_bluetooth_adapter_llsync_1.constants.DEVICE_STATE_MAP[bindStateId];
        const deviceHasBeenBind = [
            qcloud_iotexplorer_bluetooth_adapter_llsync_1.constants.DEVICE_HAS_BINDED,
            qcloud_iotexplorer_bluetooth_adapter_llsync_1.constants.DEVICE_HAS_CONNECTED,
        ].indexOf(bindState) > -1;
        if (!deviceHasBeenBind) {
            const macArr = hexArr.slice(3, 3 + 6);
            // 防止name重名
            if (device.name && device.name.indexOf('_') === -1) {
                device.name = `${device.name}_${macArr.slice(0, 2).join('')}`;
            }
            deviceProductId = hex2str(hexArr.slice(9));
        }
        else {
            deviceIdentify = hexArr.slice(3, 3 + 8).join('')
                .toLocaleLowerCase();
            deviceUserIdentify = hexArr.slice(11).join('')
                .toLocaleLowerCase();
            deviceProductId = extendInfo.productId;
        }
        if (shouldReport) {
            (_b = exports.BleComboDualModeDeviceAdapter.options.appDevSdk.reporter) === null || _b === void 0 ? void 0 : _b.info(REPORT_EVENT_TYPE, {
                message: '解析广播',
                serviceId: exports.BleComboDualModeDeviceAdapter.serviceId,
                data: {
                    bindState: bindState || 'x',
                    targetDeviceId,
                    targetDeviceIdentify,
                    // currentUserIdentify,
                    deviceUserIdentify: deviceUserIdentify || 'x',
                    deviceProductId: deviceProductId || 'x',
                    deviceIdentify: deviceIdentify || 'x',
                },
            });
        }
        const doResponse = () => {
            const deviceInfo = Object.assign(Object.assign({}, device), { 
                // 标准蓝牙的标示
                standard: true, bindState: qcloud_iotexplorer_bluetooth_adapter_llsync_1.constants.DEVICE_STATE_MAP[bindState], serviceId: exports.BleComboDualModeDeviceAdapter.serviceId, deviceName: extendInfo.deviceName || '', productId: deviceProductId, extendInfo: { moduleVersion } });
            return deviceInfo;
        };
        // 如果设备已经被绑定了，且当前为指定搜索某个设备的模式，且当前设备与目标设备、当前用户与设备归属用户都匹配，才返回，否则都不展示
        if (isSearchSpecificDeviceMode && deviceHasBeenBind && deviceIdentify === targetDeviceIdentify) {
            return doResponse();
        }
        if (!isSearchSpecificDeviceMode && !deviceHasBeenBind) {
            return doResponse();
        }
        return null;
    }
    catch (error) {
        console.error('llsync device filter error', error);
        (_c = exports.BleComboDualModeDeviceAdapter.options.appDevSdk.reporter) === null || _c === void 0 ? void 0 : _c.error(REPORT_EVENT_TYPE, {
            message: '协议广播出错',
            error,
        });
    }
};
// eslint-disable-next-line @typescript-eslint/naming-convention
exports.BleComboDualModeDeviceAdapter = ((0, qcloud_iotexplorer_bluetooth_adapter_1.AdapterDeviceAdapter4Mp)(BleComboDualModeDeviceAdapterBase));
exports.BleComboDualModeDeviceAdapter4H5 = ((0, qcloud_iotexplorer_bluetooth_adapter_1.AdapterDeviceAdapter4H5)(BleComboDualModeDeviceAdapterBase));
//# sourceMappingURL=BleComboDualModeDeviceAdapter.js.map