"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.BleComboLLSyncDeviceAdapter4H5 = exports.BleComboLLSyncDeviceAdapter = exports.BleComboLLSyncDeviceAdapterBase = exports.LLSyncComboConfig = void 0;
const tslib_1 = require("tslib");
const qcloud_iotexplorer_appdev_sdk_1 = require("qcloud-iotexplorer-appdev-sdk");
const qcloud_iotexplorer_bluetooth_adapter_1 = require("qcloud-iotexplorer-bluetooth-adapter");
const qcloud_iotexplorer_bluetooth_adapter_llsync_1 = require("qcloud-iotexplorer-bluetooth-adapter-llsync");
const qcloud_iotexplorer_logger_1 = tslib_1.__importDefault(require("qcloud-iotexplorer-logger"));
const shortid_for_miniprogram_1 = tslib_1.__importDefault(require("shortid-for-miniprogram"));
const constants_1 = require("./constants");
const { hex2str, U16ToHexString, U8ToHexString } = qcloud_iotexplorer_bluetooth_adapter_llsync_1.utils;
const { arrayBufferToHexStringArray } = qcloud_iotexplorer_bluetooth_adapter_1.blueToothHelper;
const { byteUtil, envDetect } = qcloud_iotexplorer_appdev_sdk_1.AppDevSdk.utils;
exports.LLSyncComboConfig = {
    waitGetDeviceInfoTime: 10000,
    waitSetWiFiModeTime: 10000,
    waitSetWiFiInfoTime: 10000,
    waitSetWiFiConnectTime: 70000,
    // 发送token给设备阶段：设备收到E4时会跟云端交互完成后才发送结果给小程序，网络状态差的话，很可能超过10s，因此目前超时时间延长到60s
    waitSetWiFiTokenTime: 60000,
    waitDevLogInfoTime: 10000,
};
class BleComboLLSyncDeviceAdapterBase extends qcloud_iotexplorer_bluetooth_adapter_llsync_1.LLSyncDeviceAdapterBase {
    constructor() {
        super(...arguments);
        this.reporter = {
            info: (actionName = '', _a = {}) => {
                var _b;
                var { message = '' } = _a, params = tslib_1.__rest(_a, ["message"]);
                // return this.
                const { explorerDeviceId, deviceId, isConnected } = this;
                (_b = exports.BleComboLLSyncDeviceAdapter.options) === null || _b === void 0 ? void 0 : _b.appDevSdk.reporter.info(qcloud_iotexplorer_bluetooth_adapter_llsync_1.constants.REPORT_EVENT_TYPE, {
                    serviceId: exports.BleComboLLSyncDeviceAdapter.serviceId,
                    message: message || (actionName && qcloud_iotexplorer_bluetooth_adapter_llsync_1.constants.ACTION_DESC[actionName]
                        ? `${actionName}(${qcloud_iotexplorer_bluetooth_adapter_llsync_1.constants.ACTION_DESC[actionName]})`
                        : actionName),
                    timeCost: params.timeCost || 0,
                    action: actionName,
                    data: Object.assign({ deviceId: explorerDeviceId, bleDeviceId: deviceId, isConnected }, params),
                });
            },
            // @ts-ignore
            error: (eventName = '', _a = {}) => {
                var _b;
                var { error } = _a, params = tslib_1.__rest(_a, ["error"]);
                const { explorerDeviceId, deviceId, isConnected } = this;
                // 补充errorMsg
                if (error.code && qcloud_iotexplorer_bluetooth_adapter_llsync_1.constants.ERROR_MESSAGES[error.code]) {
                    error.msg = qcloud_iotexplorer_bluetooth_adapter_llsync_1.constants.ERROR_MESSAGES[error.code];
                }
                (_b = exports.BleComboLLSyncDeviceAdapter.options) === null || _b === void 0 ? void 0 : _b.appDevSdk.reporter.error(qcloud_iotexplorer_bluetooth_adapter_llsync_1.constants.REPORT_EVENT_TYPE, {
                    // 优化message的显示
                    message: (eventName
                        && qcloud_iotexplorer_bluetooth_adapter_llsync_1.constants.ERROR_MESSAGES[eventName]
                        && `${eventName}(${qcloud_iotexplorer_bluetooth_adapter_llsync_1.constants.ERROR_MESSAGES[eventName]}：${error
                            && (error.message || error.errMsg || error.msg || error.code)})`),
                    // @ts-ignore
                    timeCost: params.timeCost || 0,
                    action: eventName,
                    error,
                    data: Object.assign(Object.assign({ deviceId: explorerDeviceId, bleDeviceId: deviceId, isConnected }, params), error),
                });
            },
        };
        this.flowId = '';
        this.comboLogger = qcloud_iotexplorer_logger_1.default.derive(() => ({
            event: 'ble.combo.llsyncble.*',
            context: {
                productId: this.productId,
                deviceName: this.deviceName,
                bleDeviceId: this.deviceId,
                flowId: this.flowId,
            },
        }));
    }
    static injectOptions(options) {
        exports.BleComboLLSyncDeviceAdapter.options = options;
    }
    notifyMessage({ type, data } = {}) {
        if (type !== 'unknown') {
            console.log('check this in notifyMessage', this, { type, data });
            return this.emit(type, { type, data });
        }
    }
    // 重写连接
    async connectDevice(params) {
        this.flowId = (0, shortid_for_miniprogram_1.default)();
        await super.connectDevice(params);
        await this.afterConnectDevice();
    }
    async afterConnectDevice() {
        // 获取设备信息
        // 设备信息中的 mtu 为 ATT_MTU
        const { version, mtu: rawDeviceDesiredMtu, needSetMtu, deviceName } = await this.getDeviceInfo();
        this.deviceName = deviceName;
        this.bleVersion = version;
        // 限制 MTU 的最小值
        const deviceDesiredMtu = Math.max(constants_1.MIN_ATT_MTU, rawDeviceDesiredMtu);
        this.mtu = (0, constants_1.attMtuToLLSyncMtu)(deviceDesiredMtu);
        this.comboLogger.info('connectDevice', {
            data: {
                version,
                mtu: rawDeviceDesiredMtu,
                needSetMtu,
                deviceName,
            },
        });
        const mtuBeforeSet = await this.tryGetMtu();
        let mtuAfterSet = null;
        let setMtuError = undefined;
        // 设置mtu的值
        if (needSetMtu && !envDetect.isIOS) {
            try {
                await this.setMtu(deviceDesiredMtu);
                this.reporter.info('mtu.set', {
                    message: `设置 MTU = ${deviceDesiredMtu} 成功`,
                    data: {
                        mtu: deviceDesiredMtu,
                    },
                    metric: {
                        type: 'success',
                    },
                });
            }
            catch (error) {
                setMtuError = error;
                this.reporter.error('mtu.set', {
                    message: `设置 MTU = ${deviceDesiredMtu} 失败`,
                    error,
                    data: {
                        mtu: deviceDesiredMtu,
                    },
                    metric: {
                        type: 'fail',
                    },
                });
                if (error.mtu === null && (error.errno === 1500104 || error.errCode === -1)) {
                    // 兼容一些主动设置 MTU 会失败的设备（联壹）
                    // 后续这个路径需要增加 productId 白名单
                    this.mtu = (0, constants_1.attMtuToLLSyncMtu)(deviceDesiredMtu);
                }
                else {
                    // 其他情况使用最小 MTU
                    this.mtu = (0, constants_1.attMtuToLLSyncMtu)(constants_1.MIN_ATT_MTU);
                }
            }
            mtuAfterSet = await this.tryGetMtu();
            this.comboLogger.info('mtu.negotiate', {
                message: [
                    `MTU 协商结果: 期望值 = ${deviceDesiredMtu}; 过程 ${mtuBeforeSet} -> ${mtuAfterSet};`,
                    `最终值 = ${(0, constants_1.llSyncMtuToAttMtu)(this.mtu)}`,
                ].join(' '),
                data: {
                    mtuBeforeSet,
                    mtuAfterSet,
                    desiredMtu: deviceDesiredMtu,
                    actualMtu: (0, constants_1.llSyncMtuToAttMtu)(this.mtu),
                },
                error: setMtuError,
            });
        }
        else {
            this.comboLogger.info('mtu.direct', {
                message: `直接使用设备提供的 MTU = ${deviceDesiredMtu}; 实际 MTU = ${mtuBeforeSet}`,
                data: {
                    desiredMtu: deviceDesiredMtu,
                    actualMtu: mtuBeforeSet,
                },
            });
        }
        if (needSetMtu) {
            this.writeMtuResult(setMtuError ? 'fail' : 'success');
        }
        // 返回设备name，后座绑定用
        return deviceName;
    }
    getDeviceInfo() {
        return this.writeAndWait4Response(qcloud_iotexplorer_bluetooth_adapter_llsync_1.constants.DEVICE_INFO_WRITE_PREFIX[qcloud_iotexplorer_bluetooth_adapter_llsync_1.constants.GET_DEVICE_INFO], qcloud_iotexplorer_bluetooth_adapter_llsync_1.constants.DEVICE_INFO, (data) => {
            // console.log('----', data);
            if (!data.length) {
                throw {
                    code: qcloud_iotexplorer_bluetooth_adapter_llsync_1.constants.DEVICE_INFO_INVALID,
                };
            }
            // 解析data消息
            const version = parseInt(data.slice(2, 3).join(''), 16);
            const mtuFiled = parseInt(data.slice(3, 5).join(''), 16);
            const needSetMtu = !!(mtuFiled >> 15);
            const mtu = mtuFiled & 0x1fff;
            const deviceNameLength = parseInt(data[5], 16);
            const deviceName = hex2str(data.slice(6).join(''));
            if (deviceNameLength !== deviceName.length) {
                throw {
                    code: qcloud_iotexplorer_bluetooth_adapter_llsync_1.constants.DEVICE_INFO_INVALID,
                };
            }
            return {
                version,
                mtu,
                needSetMtu,
                deviceName,
            };
        }, {
            timeout: exports.LLSyncComboConfig.waitGetDeviceInfoTime,
            timeoutCode: qcloud_iotexplorer_bluetooth_adapter_llsync_1.constants.WAIT_GET_DEVICE_INFO_TIMEOUT,
            writeId: qcloud_iotexplorer_bluetooth_adapter_llsync_1.constants.DEVICE_INFO_WRITE_ID,
        });
    }
    /**
     * @description WiFi模式设置
     * @returns boolean 表示设置WiFi模式成功或者失败
     */
    setWiFiMode(mode = qcloud_iotexplorer_bluetooth_adapter_llsync_1.constants.STA_WIFI_MODE) {
        const data = `${qcloud_iotexplorer_bluetooth_adapter_llsync_1.constants.DEVICE_INFO_WRITE_PREFIX[qcloud_iotexplorer_bluetooth_adapter_llsync_1.constants.SET_WIFI_MODE]}${U8ToHexString(qcloud_iotexplorer_bluetooth_adapter_llsync_1.constants.WIFI_MODE_MAP[mode])}`;
        return this.writeAndWait4Response(data, qcloud_iotexplorer_bluetooth_adapter_llsync_1.constants.SET_WIFI_MODE_RESULT, (data) => {
            // console.log('----', data);
            if (!data.length) {
                throw {
                    code: qcloud_iotexplorer_bluetooth_adapter_llsync_1.constants.SET_WIFI_MODE_RESULT_INVALID,
                };
            }
            // 解析data消息
            const result = parseInt(data.slice(2, 3).join(''), 16);
            return !result;
        }, {
            timeout: exports.LLSyncComboConfig.waitSetWiFiModeTime,
            timeoutCode: qcloud_iotexplorer_bluetooth_adapter_llsync_1.constants.SET_WIFI_MODE_RESULT_TIMEOUT,
            writeId: qcloud_iotexplorer_bluetooth_adapter_llsync_1.constants.DEVICE_INFO_WRITE_ID,
        });
    }
    /**
     *
     * @param params0 {Object} 配网的WiFi信息
     * @param parmas0.ssid {String} WiFi的ssid
     * @param params0.password {String} WiFi的密码
     * @returns {boolean} 传输WiFi结果
     */
    setWiFiInfo({ ssid, password, }) {
        const ssidHexArray = byteUtil.hexString2hexArray(byteUtil.byteArrayToHex(byteUtil.getBytesByString(ssid))) || [];
        // password可能为空
        const passwordHexArray = byteUtil.hexString2hexArray(byteUtil.byteArrayToHex(byteUtil.getBytesByString(password)))
            || [];
        console.log('----data----', {
            ssid,
            password,
            ssidHexArray,
            passwordHexArray,
        });
        const data = this.sliceData([
            qcloud_iotexplorer_bluetooth_adapter_llsync_1.constants.DEVICE_INFO_WRITE_PREFIX[qcloud_iotexplorer_bluetooth_adapter_llsync_1.constants.SET_WIFI_INFO],
            U16ToHexString(ssidHexArray.length + passwordHexArray.length),
            U8ToHexString(ssidHexArray.length),
            ...ssidHexArray,
            U8ToHexString(passwordHexArray.length),
            ...passwordHexArray,
        ], [
            [U8ToHexString(ssidHexArray.length), ...ssidHexArray],
            [U8ToHexString(passwordHexArray.length), ...passwordHexArray],
        ], qcloud_iotexplorer_bluetooth_adapter_llsync_1.constants.SET_WIFI_INFO);
        return this.writeAndWait4Response(data, qcloud_iotexplorer_bluetooth_adapter_llsync_1.constants.SET_WIFI_INFO_RESULT, (data) => {
            // console.log('----', data);
            if (!data.length) {
                throw {
                    code: qcloud_iotexplorer_bluetooth_adapter_llsync_1.constants.SET_WIFI_INFO_RESULT_INVALID,
                };
            }
            // 解析data消息
            const result = parseInt(data.slice(2, 3).join(''), 16);
            return !result;
        }, {
            timeout: exports.LLSyncComboConfig.waitSetWiFiInfoTime,
            timeoutCode: qcloud_iotexplorer_bluetooth_adapter_llsync_1.constants.SET_WIFI_INFO_RESULT_TIMEOUT,
            writeId: qcloud_iotexplorer_bluetooth_adapter_llsync_1.constants.DEVICE_INFO_WRITE_ID,
        });
    }
    /**
     * @description 下发WiFi连接请求，并获取连接状态
     * @returns {boolean} WiFi连接状态
     */
    sendConnectWiFiAndGetWiFiConnectState() {
        const data = `${qcloud_iotexplorer_bluetooth_adapter_llsync_1.constants.DEVICE_INFO_WRITE_PREFIX[qcloud_iotexplorer_bluetooth_adapter_llsync_1.constants.SET_WIFI_CONNECT]}`;
        return this.writeAndWait4Response(data, qcloud_iotexplorer_bluetooth_adapter_llsync_1.constants.SET_WIFI_CONNECT_RESULT, (data) => {
            // console.log('----', data);
            if (!data.length) {
                throw {
                    code: qcloud_iotexplorer_bluetooth_adapter_llsync_1.constants.SET_WIFI_CONNECT_RESULT_INVALID,
                };
            }
            // 解析data消息,	Stattion状态表示BLE设备作为WiFi Station模式的工作状态，0x0表示连接，其他表示非连接。
            const stationState = parseInt(data.slice(3, 4).join(''), 16);
            const ssidLength = parseInt(data.slice(5, 6).join(''), 16);
            let ssid = '';
            if (ssidLength) {
                ssid = hex2str(data.slice(6, 6 + ssidLength));
            }
            return {
                connected: !stationState,
                ssid,
            };
        }, {
            timeout: exports.LLSyncComboConfig.waitSetWiFiConnectTime,
            timeoutCode: qcloud_iotexplorer_bluetooth_adapter_llsync_1.constants.SET_WIFI_CONNECT_RESULT_TIMEOUT,
            writeId: qcloud_iotexplorer_bluetooth_adapter_llsync_1.constants.DEVICE_INFO_WRITE_ID,
        });
    }
    sendToken({ token, }) {
        const tokenHexArray = byteUtil.hexString2hexArray(byteUtil.byteArrayToHex(byteUtil.getBytesByString(token)));
        console.log('----data----', {
            token,
            tokenHexArray,
        });
        const data = this.sliceData([
            qcloud_iotexplorer_bluetooth_adapter_llsync_1.constants.DEVICE_INFO_WRITE_PREFIX[qcloud_iotexplorer_bluetooth_adapter_llsync_1.constants.SET_WIFI_TOKEN],
            U16ToHexString(tokenHexArray.length),
            ...tokenHexArray,
        ], [tokenHexArray], qcloud_iotexplorer_bluetooth_adapter_llsync_1.constants.SET_WIFI_TOKEN);
        return this.writeAndWait4Response(data, qcloud_iotexplorer_bluetooth_adapter_llsync_1.constants.SET_WIFI_TOKEN_RESULT, (data) => {
            // console.log('----', data);
            if (!data.length) {
                throw {
                    code: qcloud_iotexplorer_bluetooth_adapter_llsync_1.constants.SET_WIFI_TOKEN_RESULT_INVALID,
                };
            }
            // 解析data消息
            const result = parseInt(data.slice(2, 3).join(''), 16);
            return !result;
        }, {
            timeout: exports.LLSyncComboConfig.waitSetWiFiTokenTime,
            timeoutCode: qcloud_iotexplorer_bluetooth_adapter_llsync_1.constants.SET_WIFI_TOKEN_RESULT_TIMEOUT,
            writeId: qcloud_iotexplorer_bluetooth_adapter_llsync_1.constants.DEVICE_INFO_WRITE_ID,
        });
    }
    getModuleLog() {
        // 获取错误日志吧
        const data = `${qcloud_iotexplorer_bluetooth_adapter_llsync_1.constants.DEVICE_INFO_WRITE_PREFIX[qcloud_iotexplorer_bluetooth_adapter_llsync_1.constants.GET_DEV_LOG]}`;
        return this.writeAndWait4Response(data, qcloud_iotexplorer_bluetooth_adapter_llsync_1.constants.GET_DEV_LOG_INFO, (data) => {
            if (!data.length) {
                throw {
                    code: qcloud_iotexplorer_bluetooth_adapter_llsync_1.constants.GET_DEV_LOG_INFO_INVALID,
                };
            }
            // 解析data消息
            const length = parseInt(data.slice(1, 3).join(''), 16);
            const logStr = hex2str(data.slice(4, 4 + length));
            return {
                logStr,
            };
        }, {
            timeout: exports.LLSyncComboConfig.waitDevLogInfoTime,
            timeoutCode: qcloud_iotexplorer_bluetooth_adapter_llsync_1.constants.GET_DEV_LOG_INFO_TIMEOUT,
            writeId: qcloud_iotexplorer_bluetooth_adapter_llsync_1.constants.DEVICE_INFO_WRITE_ID,
        });
    }
    async tryGetMtu() {
        try {
            const { mtu } = await this.getBLEMTU();
            return mtu;
        }
        catch (error) {
            return null;
        }
    }
}
exports.BleComboLLSyncDeviceAdapterBase = BleComboLLSyncDeviceAdapterBase;
BleComboLLSyncDeviceAdapterBase.options = null;
BleComboLLSyncDeviceAdapterBase.serviceId16 = '0000FFF0-0000-1000-8000-00805F9B34FB'; // 搜索时匹配的serviceId
BleComboLLSyncDeviceAdapterBase.serviceId = '0000FFF0-65D0-4E20-B56A-E493541BA4E2'; // 服务的serviceId
BleComboLLSyncDeviceAdapterBase.deviceFilter = (device) => {
    var _a, _b, _c;
    if (!device.advertisServiceUUIDs
        || !device.advertisServiceUUIDs.find(id => id === exports.BleComboLLSyncDeviceAdapter.serviceId16)
        || !device.advertisData) {
        return null;
    }
    try {
        // 计算是否要上报，这里取样1/10
        const needReport = Math.floor(Math.random() * 10) % 10 === 1 || true;
        const hexArr = arrayBufferToHexStringArray(device.advertisData);
        if (needReport) {
            (_a = exports.BleComboLLSyncDeviceAdapter.options) === null || _a === void 0 ? void 0 : _a.appDevSdk.reporter.info(qcloud_iotexplorer_bluetooth_adapter_llsync_1.constants.REPORT_EVENT_TYPE, {
                message: '收到广播',
                serviceId: exports.BleComboLLSyncDeviceAdapter.serviceId,
                hexArr: hexArr.join(','),
            });
        }
        const moduleVersion = parseInt(hexArr[2], 16) >> 4;
        let deviceProductId = '';
        const macArr = hexArr.slice(3, 3 + 6);
        let name = device.localName || device.name;
        if (name && name.indexOf('_') === -1) {
            name = `${name}_${macArr.slice(0, 2).join('')}`;
        }
        deviceProductId = hex2str(hexArr.slice(9));
        if (needReport) {
            (_b = exports.BleComboLLSyncDeviceAdapter.options) === null || _b === void 0 ? void 0 : _b.appDevSdk.reporter.info(qcloud_iotexplorer_bluetooth_adapter_llsync_1.constants.REPORT_EVENT_TYPE, {
                message: '解析广播',
                serviceId: exports.BleComboLLSyncDeviceAdapter.serviceId,
                data: {
                    deviceProductId: deviceProductId || 'x',
                },
            });
        }
        return Object.assign(Object.assign({}, device), { 
            // 标准蓝牙的blecombo的标示
            standardBleCombo: true, serviceId: exports.BleComboLLSyncDeviceAdapter.serviceId, deviceName: '', productId: deviceProductId, extendInfo: { moduleVersion }, name });
    }
    catch (error) {
        (_c = exports.BleComboLLSyncDeviceAdapter.options) === null || _c === void 0 ? void 0 : _c.appDevSdk.reporter.error(qcloud_iotexplorer_bluetooth_adapter_llsync_1.constants.REPORT_EVENT_TYPE, {
            message: '协议广播出错',
            error,
        });
    }
};
// eslint-disable-next-line @typescript-eslint/naming-convention
exports.BleComboLLSyncDeviceAdapter = ((0, qcloud_iotexplorer_bluetooth_adapter_1.AdapterDeviceAdapter4Mp)(BleComboLLSyncDeviceAdapterBase));
exports.BleComboLLSyncDeviceAdapter4H5 = ((0, qcloud_iotexplorer_bluetooth_adapter_1.AdapterDeviceAdapter4H5)(BleComboLLSyncDeviceAdapterBase));
//# sourceMappingURL=BleComboLLSyncDeviceAdapter.js.map