"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.BleComboLLSyncTask = void 0;
const constants_1 = require("./constants");
const BleComboTask_1 = require("../BleComboTask");
class BleComboLLSyncTask extends BleComboTask_1.BleComboTask {
    constructor(options) {
        super(options);
    }
    async setWiFiMode() {
        if (!this.deviceAdapter) {
            throw new Error('deviceAdapter option is not provided to this plugin');
        }
        const success = await this.deviceAdapter.setWiFiMode();
        if (!success) {
            throw {
                code: 'BLE_SET_WIFI_MODE_ERROR',
            };
        }
    }
    async sendWiFiInfo() {
        const success = await this.deviceAdapter.setWiFiInfo({
            ssid: this.ssid,
            password: this.password,
        });
        if (!success) {
            throw {
                code: 'BLE_SEND_WIFI_INFO_ERROR',
                data: {
                    ssid: this.ssid,
                    password: this.password,
                },
            };
        }
    }
    async connectWifi() {
        try {
            const { connected, ssid: ssidData } = await this.deviceAdapter.sendConnectWiFiAndGetWiFiConnectState();
            console.log('----ssidData', ssidData);
            if (connected && ssidData.length && ssidData === this.ssid) {
                return ssidData;
            }
            else {
                throw {
                    code: 'BLE_WIFI_CONNECTION_ERROR',
                    connected,
                    ssidData,
                };
            }
        }
        catch (e) {
            console.error(e);
            return Promise.reject(Object.assign({ msg: constants_1.BLE_COMBO_ERRORS[e.code] || '' }, e));
        }
    }
    async sendToken() {
        try {
            const success = await this.deviceAdapter.sendToken({
                token: this.token,
            });
            if (!success) {
                throw {
                    code: 'BLE_SEND_TOKEN_ERROR',
                    token: this.token,
                };
            }
            return {
                productId: this.deviceAdapter.productId,
                deviceName: this.deviceAdapter.deviceName,
            };
        }
        catch (e) {
            console.error(e);
            return Promise.reject(Object.assign({ msg: constants_1.BLE_COMBO_ERRORS[e.code] || '' }, e));
        }
    }
}
exports.BleComboLLSyncTask = BleComboLLSyncTask;
//# sourceMappingURL=BleComboLLSyncTask.js.map