"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.BleComboTask = void 0;
const qcloud_iotexplorer_appdev_sdk_1 = require("qcloud-iotexplorer-appdev-sdk");
const { delay, } = qcloud_iotexplorer_appdev_sdk_1.AppDevSdk.utils;
class BleComboTask extends qcloud_iotexplorer_appdev_sdk_1.AppDevSdk.utils.EventEmitter {
    constructor({ targetWifiInfo, wifiConfToken = '', deviceAdapter, reporter, wifiConfType, flowLogger, }) {
        super();
        this.sequence = 0x00;
        this.sendDataInterval = 300; // 发送数据的间隔
        const { SSID: ssid, password, } = targetWifiInfo;
        this.ssid = ssid;
        this.password = password || '';
        this.token = wifiConfToken;
        this.deviceAdapter = deviceAdapter;
        this.logger = reporter;
        this.processing = false;
        this.wifiConfType = wifiConfType;
        this.flowLogger = flowLogger;
        this.flowLogger.info('ssid', `ssid = ${ssid}`, {
            ssid,
            // 去掉敏感内容
            password: password.replace(/^\w{3}/, '***'),
        });
    }
    async start() {
        if (this.processing) {
            return Promise.reject({
                code: 'PROCESSING_PLEASE_WAIT',
            });
        }
        this.processing = true;
        await this.flowLogger
            .step('setWiFiMode', 'setWiFiMode', true)
            .wrap(this.setWiFiMode());
        await delay(this.sendDataInterval);
        await this.flowLogger
            .step('sendWiFiInfo', 'sendWiFiInfo', true)
            .wrap(this.sendWiFiInfo());
        await delay(this.sendDataInterval);
        await this.flowLogger
            .step('connectWifi', 'connectWifi', true)
            .wrap(this.connectWifi());
        await delay(this.sendDataInterval);
    }
    interrupt() {
        this.processing = false;
        // 断开蓝牙连接
        this.deviceAdapter.disconnectDevice();
    }
}
exports.BleComboTask = BleComboTask;
//# sourceMappingURL=BleComboTask.js.map