import { AppDevSdk } from 'qcloud-iotexplorer-appdev-sdk';
import { DATA_MODE, SEND_TOKEN_SUB_TYPE } from './constants';
import { BleComboTask } from '../BleComboTask';
const { byteUtil, delay, } = AppDevSdk.utils;
export class BleComboEspTask extends BleComboTask {
    constructor(options) {
        super(options);
    }
    async setWiFiMode() {
        const switchData = await this.deviceAdapter.sendSwitchWiFi();
        this.logger.info('sendSwitchWiFiSucc', {
            bleData: switchData,
        });
    }
    async sendWiFiInfo() {
        const ssidData = await this.deviceAdapter.sendSsid(this.ssid);
        this.logger.info('sendSsidSucc', {
            bleData: ssidData,
        });
        await delay(this.sendDataInterval);
        const passwordData = await this.deviceAdapter.sendPassword(this.password);
        this.logger.info('sendPasswordSucc', {
            bleData: passwordData,
        });
    }
    async connectWifi() {
        try {
            const ssidData = await this.deviceAdapter.getWifiConnectState();
            const correctSsidData = byteUtil.byteArrayToHex(byteUtil.getBytesByString(this.ssid)).toUpperCase();
            console.log('----ssidData', ssidData);
            if (ssidData.length && ssidData.join('') === correctSsidData) {
                return ssidData;
            }
            else {
                throw {
                    code: 'BLE_WIFI_CONNECTION_ERROR',
                    ssidData: ssidData.join(''),
                    correctSsidData,
                };
            }
        }
        catch (e) {
            console.error(e);
            return Promise.reject(Object.assign(Object.assign({}, e), { code: `BLE_WIFI_CONNECTION_${e.code || 'ERROR'}` }));
        }
    }
    async sendToken() {
        try {
            const deviceData = await this.deviceAdapter.getCustomData({
                mode: DATA_MODE,
                subType: SEND_TOKEN_SUB_TYPE,
                data: this.token,
            });
            this.logger.info('deviceData', { deviceData });
            // 流程结束的，断开蓝牙连接
            this.interrupt();
            return {
                productId: deviceData.productId,
                deviceName: deviceData.deviceName,
            };
        }
        catch (e) {
            return Promise.reject(Object.assign({ code: `BLE_TOKEN_STATE_${e.code || 'ERROR'}` }, e));
        }
    }
}
//# sourceMappingURL=BleComboEspTask.js.map